;
;REVISO: 04-12-2003
;DATA: 20-10-2003
;Autor: Marcos Vonnilmam
;Projeto: Driver "Microsteping PWM" para controlar STEP MOTOR - (5 ou 6 fios).
;Verso: 1.00.00
;Cristal Oscilador: 20 Mhz
;Projeto aprovado para 10.000 steps pr volta "360 Graus", step motor 1,8 graus
;
;*********************************** DIRETIVAS DO COMPILADOR ************************************

	#include <p16f877.inc>            ; Chip utilizado em produo.           
	#include <macros.asm>

	ERRORLEVEL	    -302                       
	ERRORLEVEL	    -306                     

	radix	dec

;*********************************** MAPEAMENTO DE MEMRIA **************************************
;NOTA: A posio 0x70  de uso exclusivo do iCD 1.
						                      
	cblock	        0x20      
	vap1			;registro de SENO
	vbp1			;registro de COSENO
	pwm1			;contador PWM
	acc1			;acumulador pseudo
	flag
	port_pwm
	endc

	cblock	        0x71    ;estes registros esto em lugar extrategico  
	w_temp                  ;registros de "guarda" para a interrupo.
	pclath_temp             ;note que esto espelhados em todos os banks
	status_temp
	fsr_temp     
	endc

;********************************* MAPA DE DEFINIES GLOBAIS ***********************************

#define	        ap1              port_pwm,0	; Bobina +A		
#define	        an1              port_pwm,1	; Bobina -A
#define	        bp1              port_pwm,2	; Bobina +B
#define		bn1    		 port_pwm,3	; Bobina -B
#define		step_dir	 portb,7	; direo do motor 1
#define		step_mot	 portb,6	; entrada de pulsos para os passos.
#define		flag_step_mot	 flag,0
#define		motor1		 porta

;************************************ CONFIGURAO INICIAL **************************************
								
	org     0x00       	
	clrf    pclath          
	goto    init_config     
	org     0x04            ; Vetor interrupo.     
;        push
	bank0
	clrf	pclath	
;;;	goto    interrupt
	retfie

init_config:
	bank1 
	movlw   b'00000000'     ;                       
	movwf   trisa           ;              
	movlw   b'11111111'     ;	
	movwf   trisb           ;
   	movlw   b'00000000'     ;prescaler: 000=002 001=4 010=8 011=16 100=32 101=64 110=128 111=256
   	movwf   option_reg      
	movlw	b'00000110'
	movwf	adcon1
 	bank0
  	bcf	intcon, gie	;dasabilita interrupo global.(s habilita aps iniciar a BiOS)
	goto    bios
;---------------------------------

;***************
;* 10000 Steps *
;***************	190,194,196

tab_sin:
	addwf	pcl, f

	dt	50,52,53,55,56,58,59,61,62,64,65,67,68,70,71,73,74,75,77,78,79,81,82,83,84,85,86,88,89,90,90
	dt	91,92,93,94,95,95,96,96,97,98,98,98,99,99,99,100,100,100,100,100,100,100,99,99,99,98,98,98 
	dt	97,96,96,95,95,94,93,92,91,90,90,89,88,86,85,84,83,82,81,79,78,77,75,74,73,71,70,68,67,65,64 
	dt	62,61,59,58,56,55,53,52,50,48,47,45,44,42,41,39,38,36,35,33,32,30,29,27,26,25,23,22,21,19,18,17 
	dt	16,15,14,12,11,10,10,9,8,7,6,5,5,4,4,3,2,2,2,1,1,1,0,0,0,0,0,0,0,1,1,1,2,2,3,4,4,5,5,6,7,8,9
	dt	10,10,11,12,14,15,16,17,18,19,21,22,23,25,26,27,29,30,32,33,35,36,38,39,41,42,44,45,47,48
;--------------------------

bios:
	clrf 	pwm1
	movlw	50
	movwf	vap1
	movlw	100
	movwf	vbp1
	bsf	flag_step_mot

;*****************
;* Modulador PWM *
;*****************

inic:
	bcf	ap1
	bsf	an1
	bcf	bp1
	bsf	bn1
	movf	port_pwm, w
	movwf	motor1
	call	load_pwm
	movlw	100
	movwf	acc1
loop_pwm:
	movf	acc1, w
	subwf	vap1, w
	btfss	status, z
	goto	mot1b
	bsf	ap1
	bcf	an1
	movf	port_pwm, w
	movwf	motor1
mot1b:
	movf	acc1, w
	subwf	vbp1, w
	btfss	status, z
	goto	dec_pwm
	bsf	bp1
	bcf	bn1
	movf	port_pwm, w
	movwf	motor1
dec_pwm:
	decfsz	acc1, f
	goto	loop_pwm
	goto	inic
;--------------------------------

;****************
;* Atualiza PWM *
;****************

load_pwm:
	btfss	step_mot
	goto	$+5
	btfsc	flag_step_mot
	return
	bsf	flag_step_mot
	goto	$+3
	bcf	flag_step_mot
	return

	movf	pwm1, w
	call	tab_sin
	movwf	vap1
	movf	pwm1, w
	pag2
	call	tab_cos
	pag1
	movwf	vbp1

	btfsc	step_dir
	goto	inc_step
	goto	dec_step
inc_step:
	incf	pwm1, f
	movlw	190
	subwf	pwm1, w
	btfss	status, z
	return
	clrf	pwm1
	return
dec_step:
	decf	pwm1, f
	movlw	255
	subwf	pwm1, w
	btfss	status, z
	return	
	movlw	189
	movwf	pwm1
	return

;*pagina 1*

	org	.2048

tab_cos:
	addwf	pcl, f

 	dt 	100,100,100,100,99,99,99,98,98,98,97,96,96,95,95,94,93,92,91,90,90,89,88,86,85 
	dt	84,83,82,81,79,78,77,75,74,73,71,70,68,67,65,64,62,61,59,58,56,55,53,52,50,48,47,45,44,42,41,39 
	dt	38,36,35,33,32,30,29,27,26,25,23,22,21,19,18,17,16,15,14,12,11,10,10,9,8,7,6,5,5,4,4,3,2,2,2
	dt	1,1,1,0,0,0,0,0,0,0,1,1,1,2,2,3,4,4,5,5,6,7,8,9,10,10,11,12,14,15,16,17,18,19,21,22,23
	dt	25,26,27,29,30,32,33,35,36,38,39,41,42,44,45,47,48,50,52,53,55,56,58,59,61,62,64,65,67,68,70,71
	dt	73,74,75,77,78,79,81,82,83,84,85,86,88,89,90,90,91,92,93,94,95,95,96,96,97,98,98,98,99,99,99,100,100,100
;--------------------------------

	end
;--------------------------------
;**************
;* 2000 Steps *
;**************		40
tab_sin: 
	addwf	pcl, f

	dt	.50,.58,.65,.73,.79,.85,.90,.95,.98,.99,.100,.99,.98,.95,.90,.85,.79,.73,.65,.58,.50,.42 
  	dt	.35,.27,.21,.15,.10,.5,.2,.1,.0,.1,.2,.5,.10,.15,.21,.27,.35,.42        
tab_cos:
	addwf	pcl, f
	
	dt	.100,.99,.98,.95,.90,.85,.79,.73,.65,.58,.50,.42,.35,.27,.21,.15,.10,.5,.2,.1,.0,.1
	dt	.2,.5,.10,.15,.21,.27,.35,.42,.50,.58,.65,.73,.79,.85,.90,.95,.98,.99 

;---------------------------------
;***************
;* 2.400 Steps *
;***************	44

tab_sin:
	addwf	pcl, f

	dt 	.50,.57,.63,.69,.75,.80,.85,.90,.93,.96,.98,.100,.98,.96,.93,.90,.85,.80,.75,.69,.63,.57,.50 
	dt 	.43,.37,.31,.25,.20,.15,.10,.7,.4,.2,.0,.2,.4,.7,.10,.15,.20,.25,.31,.37,.43 

tab_cos:
	addwf	pcl, f

	dt 	.100,.98,.96,.93,.90,.85,.80,.75,.69,.63,.57,.50,.43,.37,.31,.25,.20,.15,.10,.7,.4,.2
	dt 	.0,.2,.4,.7,.10,.15,.20,.25,.31,.37,.43,.50,.57,.63,.69,.75,.80,.85,.90,.93,.96,.98      

;***************
;* 4.000 Steps *
;***************	76

tab_sin:
	addwf	pcl, f

 	dt 	50,54,58,62,65,69,73,76,79,82,85,88,90,93,95,96,98,99,99,100,99,99,98 
 	dt 	96,95,93,90,88,85,82,79,76,73,69,65,62,58,54,50,46,42,38,35,31,27,24
 	dt 	21,18,15,12,10,07,05,04,02,01,01,00,01,01,02,04,05,07,10,12,15,18,21  
 	dt 	24,27,31,35,38,42,46                                                                  

tab_cos:
	addwf	pcl, f

 	dt 	100,99,99,98,96,95,93,90,88,85,82,79,76,73,69,65,62,58,54,50,46,42,38
	dt	35,31,27,24,21,18,15,12,10,07,05,04,02,01,01,00,01,01,02,04,05,07,10  
	dt	12,15,18,21,24,27,31,35,38,42,46,50,54,58,62,65,69,73,76,79,82,85,88
	dt	90,93,95,96,98,99,99
;---------------------------------

;***************
;* 10000 Steps *
;***************	194

tab_sin:
	addwf	pcl, f

	dt	50,52,53,55,56,58,59,61,62,64,65,67,68,70,71,73,74,75,77,78,79,81,82,83,84,85,86,88,89,90,90
	dt	91,92,93,94,95,95,96,96,97,98,98,98,99,99,99,100,100,100,100,100,100,100,99,99,99,98,98,98 
	dt	97,96,96,95,95,94,93,92,91,90,90,89,88,86,85,84,83,82,81,79,78,77,75,74,73,71,70,68,67,65,64 
	dt	62,61,59,58,56,55,53,52,50,48,47,45,44,42,41,39,38,36,35,33,32,30,29,27,26,25,23,22,21,19,18,17 
	dt	16,15,14,12,11,10,10,9,8,7,6,5,5,4,4,3,2,2,2,1,1,1,0,0,0,0,0,0,0,1,1,1,2,2,3,4,4,5,5,6,7,8,9
	dt	10,10,11,12,14,15,16,17,18,19,21,22,23,25,26,27,29,30,32,33,35,36,38,39,41,42,44,45,47,48

tab_cos:
	addwf	pcl, f

 	dt 	100,100,100,100,99,99,99,98,98,98,97,96,96,95,95,94,93,92,91,90,90,89,88,86,85 
	dt	84,83,82,81,79,78,77,75,74,73,71,70,68,67,65,64,62,61,59,58,56,55,53,52,50,48,47,45,44,42,41,39 
	dt	38,36,35,33,32,30,29,27,26,25,23,22,21,19,18,17,16,15,14,12,11,10,10,9,8,7,6,5,5,4,4,3,2,2,2
	dt	1,1,1,0,0,0,0,0,0,0,1,1,1,2,2,3,4,4,5,5,6,7,8,9,10,10,11,12,14,15,16,17,18,19,21,22,23
	dt	25,26,27,29,30,32,33,35,36,38,39,41,42,44,45,47,48,50,52,53,55,56,58,59,61,62,64,65,67,68,70,71
	dt	73,74,75,77,78,79,81,82,83,84,85,86,88,89,90,90,91,92,93,94,95,95,96,96,97,98,98,98,99,99,99,100,100,100
				
